/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2019 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */

#include "config.h"

#include <genvector/vts0.h>
#include <genvector/gds_char.h>

#include <librnd/core/safe_fs.h>
#include <librnd/core/safe_fs_dir.h>

typedef struct {
	/* optional configuration */
	rnd_hidlib_t *hidlib;
	vts0_t names;
	unsigned int case_insensitive:1;

	/*	output */
	gds_t path;
	char *name_found; /* points to within ->path.array */

	/* internal states */
	DIR *dir;
	vts0_t open; /* open list of directory paths to search; always the last item is removed first */
	int dirlen;
} csch_lib_it_t;

/* Recursive search for a file in the libraries. The file may have alternate
   names. Each library is recursively searched, in order they appear on the
   libs. If a directory contains multiple file names, return each. */

/* Create a new search in it */
void csch_lib_find_alloc(csch_lib_it_t *it, rnd_hidlib_t *hidlib, vts0_t *libs, const char *cwd);

/* Add a name to the search (name is not strdup'd so must be available
   until csch_lib_find_free() */
void csch_lib_find_add_name(csch_lib_it_t *it, const char *name);

/* Find the first or next fill path; returns NULL when search ends. */
char *csch_lib_find_next(csch_lib_it_t *it);

/* Stop searching and free allocations in 'it' */
void csch_lib_find_free(csch_lib_it_t *it);



